@@ -5,6 +5,7 @@
 import sys
 import mkchromecast.__init__        # This is to verify against some needed variables
 from mkchromecast.config import *
+from mkchromecast.utils import is_installed
 import os
 import getpass
 import subprocess
@@ -20,7 +21,10 @@ tray = mkchromecast.__init__.tray
 USER = getpass.getuser()
 
 if platform == 'Darwin':
-    PATH ='./bin:./nodejs/bin:/Users/'+str(USER)+'/bin:/usr/local/bin:/usr/local/sbin:/usr/bin:/bin:/usr/sbin:/sbin:/opt/X11/bin:/usr/X11/bin:/usr/games:'+ os.environ['PATH']
+    PATH ='./bin:./nodejs/bin:/Users/' + \
+            str(USER) + \
+            '/bin:/usr/local/bin:/usr/local/sbin:/usr/bin:/bin:/usr/sbin:/sbin:/opt/X11/bin:/usr/X11/bin:/usr/games:' + \
+            os.environ['PATH']
 else:
     PATH = os.environ['PATH']
 
@@ -28,18 +32,6 @@ if debug == True:
     print('USER ='+str(USER))
     print('PATH ='+str(PATH))
 
-def is_installed(name):
-    iterate = PATH.split(':')
-    for item in iterate:
-        verifyif = str(item+'/'+name)
-        if os.path.exists(verifyif) == False:
-            continue
-        else:
-            if debug == True:
-                print('Program '+str(name)+' found in '+str(verifyif))
-            return True
-    return
-
 """
 Configparser is imported differently in Python3
 """
@@ -66,7 +58,8 @@ def ConfigSectionMap(section):
     return dict1
 
 if tray == True:
-    from PyQt5.QtWidgets import QWidget, QLabel, QComboBox, QApplication, QPushButton
+    from PyQt5.QtWidgets import (QWidget, QLabel, QComboBox, QApplication,
+            QPushButton, QLineEdit)
     from PyQt5 import QtCore
 
     class preferences(QWidget):
@@ -94,6 +87,8 @@ if tray == True:
             self.iconcolors()
             self.notifications()
             self.searchatlaunch()
+            if platform == 'Linux':
+                self.alsadevice()
             self.buttons()
             self.window()
 
@@ -105,16 +100,22 @@ if tray == True:
                 'node',
                 'ffmpeg',
                 'avconv',
-                'parec'
+                'parec',
+                'gstreamer'
                 ]
             self.backends = []
             if platform == 'Darwin':
                 for item in backends_supported:
-                    if is_installed(item) == True and item != 'avconv':
+                    if (is_installed(item, PATH, debug) == True
+                            and item != 'avconv' and item !='gstreamer'):
                         self.backends.append(item)
             else:
                 for item in backends_supported:
-                    if is_installed(item) == True and item != 'node':
+                    if (is_installed(item, PATH, debug) == True
+                            and item != 'node' and item != 'gstreamer'):
+                        self.backends.append(item)
+                    elif (is_installed('gst-launch-1.0', PATH, debug) == True and
+                            item == 'gstreamer'): # hardcoded gst-launch-1.0 for gstreamer
                         self.backends.append(item)
             backendindex = self.backends.index(self.backendconf)
             self.backend = QLabel('Select Backend', self)
@@ -132,7 +133,7 @@ if tray == True:
             Codec
             """
             self.codec = QLabel('Audio Coding Format', self)
-            self.codec.move(20*self.scale_factor, 56*self.scale_factor)
+            self.codec.move(20 * self.scale_factor, 56 * self.scale_factor)
             self.qccodec = QComboBox(self)
             self.qccodec.clear()
             if self.backendconf == 'node':
@@ -160,17 +161,14 @@ if tray == True:
             Bitrate
             """
             self.bitrate = QLabel('Select Bitrate (kbit/s)', self)
-            self.bitrate.move(20*self.scale_factor, 88*self.scale_factor)
+            self.bitrate.move(20 * self.scale_factor, 88 * self.scale_factor)
             self.qcbitrate = QComboBox(self)
             self.qcbitrate.clear()
-            self.qcbitrate.move(180*self.scale_factor, 88*self.scale_factor)
+            self.qcbitrate.move(180 * self.scale_factor, 88 * self.scale_factor)
             self.qcbitrate.setMinimumContentsLength(7)
             if self.codecconf == 'wav':
                 self.bitrates = ['None']
                 bitrateindex = 0
-            elif self.codecconf == 'flac':
-                self.bitrates = ['None']
-                bitrateindex = 0
             else:
                 self.bitrates = [
                     '128',
@@ -192,7 +190,10 @@ if tray == True:
             Sample rate
             """
             self.samplerates = [
+                '192000',
+                '176400',
                 '96000',
+                '88200',
                 '48000',
                 '44100',
                 '32000',
@@ -267,21 +268,35 @@ if tray == True:
             self.qcatlaunch.setCurrentIndex(launchindex)
             self.qcatlaunch.activated[str].connect(self.onActivatedatlaunch)
 
+        def alsadevice(self):
+            """
+            Set the ALSA Device
+            """
+            self.alsadevice = QLabel('ALSA Device', self)
+            self.alsadevice.move(20*self.scale_factor, 244*self.scale_factor)
+            self.qle = QLineEdit(self)
+            self.qle.move(179*self.scale_factor, 244*self.scale_factor)
+            self.qle.setFixedWidth(84*self.scale_factor)
+            self.read_defaults()
+            if self.alsadeviceconf != None:
+                self.qle.setText(self.alsadeviceconf)
+            self.qle.textChanged[str].connect(self.onActivatedalsadevice)
+
         def buttons(self):
             """
             Buttons
             """
             resetbtn = QPushButton("Reset Settings", self)
-            resetbtn.move(10*self.scale_factor, 252*self.scale_factor)
+            resetbtn.move(10*self.scale_factor, 274*self.scale_factor)
             resetbtn.clicked.connect(self.reset_configuration)
 
             faqbtn = QPushButton("FAQ", self)
-            faqbtn.move(138*self.scale_factor, 252*self.scale_factor)
+            faqbtn.move(138*self.scale_factor, 274*self.scale_factor)
             faqbtn.clicked.connect(lambda: webbrowser.open('https://github.com/muammar/mkchromecast/wiki/FAQ'))
 
             donbtn = QPushButton("Donate :)", self)
-            donbtn.move(204*self.scale_factor, 252*self.scale_factor)
-            donbtn.clicked.connect(lambda: webbrowser.open('https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=JQGD4UXPBS96U'))
+            donbtn.move(204*self.scale_factor, 274*self.scale_factor)
+            donbtn.clicked.connect(lambda: webbrowser.open('https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=RZLF7TDCAXT9Q&lc=US&item_name=mkchromecast&currency_code=USD&bn=PP%2dDonationsBF%3abtn_donateCC_LG%2egif%3aNonHosted'))
 
         def window(self):
             """
@@ -289,9 +304,9 @@ if tray == True:
             """
             self.setGeometry(300*self.scale_factor, 300*self.scale_factor, 300*self.scale_factor, 200*self.scale_factor)
             if platform == 'Darwin':
-                self.setFixedSize(310*self.scale_factor, 300*self.scale_factor)     #This is to fix the size of the window
+                self.setFixedSize(310*self.scale_factor, 320*self.scale_factor)     #This is to fix the size of the window
             else:
-                self.setFixedSize(282*self.scale_factor, 300*self.scale_factor)     #This is to fix the size of the window
+                self.setFixedSize(282*self.scale_factor, 320*self.scale_factor)     #This is to fix the size of the window
             self.setWindowFlags(QtCore.Qt.WindowCloseButtonHint | QtCore.Qt.WindowMinimizeButtonHint | QtCore.Qt.WindowStaysOnTopHint)
             self.setWindowTitle('Mkchromecast Preferences')
 
@@ -336,14 +351,14 @@ if tray == True:
 
         def onActivatedbk(self, text):
             self.config.read(self.configf)
-            self.config.set('settings','backend',text)
+            self.config.set('settings', 'backend', text)
             self.write_config()
             self.read_defaults()
             self.qccodec.clear()
             if self.backendconf == 'node':
                 codecs = ['mp3']
                 self.config.read(self.configf)
-                self.config.set('settings','codec','mp3')
+                self.config.set('settings', 'codec', 'mp3')
                 self.write_config()
             else:
                 codecs = [
@@ -365,16 +380,13 @@ if tray == True:
 
         def onActivatedcc(self, text):
             self.config.read(self.configf)
-            self.config.set('settings','codec',text)
+            self.config.set('settings', 'codec', text)
             self.write_config()
             self.read_defaults()
             self.qcbitrate.clear()
             if self.codecconf == 'wav':
                 bitrates = ['None']
                 bitrateindex = 0
-            elif self.codecconf == 'flac':
-                bitrateindex = 0
-                bitrates = ['None']
             else:
                 self.configurations.chk_config()
                 self.read_defaults()
@@ -396,31 +408,40 @@ if tray == True:
 
         def onActivatedbt(self, text):
             self.config.read(self.configf)
-            self.config.set('settings','bitrate',text)
+            self.config.set('settings', 'bitrate', text)
             self.write_config()
             self.read_defaults()
 
         def onActivatedsr(self, text):
             self.config.read(self.configf)
-            self.config.set('settings','samplerate',text)
+            self.config.set('settings', 'samplerate', text)
             self.write_config()
             self.read_defaults()
 
         def onActivatednotify(self, text):
             self.config.read(self.configf)
-            self.config.set('settings','notifications',text)
+            self.config.set('settings', 'notifications', text)
             self.write_config()
             self.read_defaults()
 
         def onActivatedcolors(self, text):
             self.config.read(self.configf)
-            self.config.set('settings','colors',text)
+            self.config.set('settings', 'colors', text)
             self.write_config()
             self.read_defaults()
 
         def onActivatedatlaunch(self, text):
             self.config.read(self.configf)
-            self.config.set('settings','searchatlaunch',text)
+            self.config.set('settings', 'searchatlaunch', text)
+            self.write_config()
+            self.read_defaults()
+
+        def onActivatedalsadevice(self, text):
+            self.config.read(self.configf)
+            if not text:
+                self.config.set('settings', 'alsadevice', None)
+            else:
+                self.config.set('settings', 'alsadevice', text)
             self.write_config()
             self.read_defaults()
 
@@ -429,25 +450,26 @@ if tray == True:
             self.codecconf = ConfigSectionMap('settings')['codec']
             if self.backendconf == 'node' and self.codecconf != 'mp3':
                 self.config.read(self.configf)
-                self.config.set('settings','codec','mp3')
+                self.config.set('settings', 'codec', 'mp3')
                 self.write_config()
                 self.codecconf = ConfigSectionMap('settings')['codec']
             self.bitrateconf = ConfigSectionMap('settings')['bitrate']
             self.samplerateconf = ConfigSectionMap('settings')['samplerate']
             self.notificationsconf = ConfigSectionMap('settings')['notifications']
-            self.searchatlaunchconf = ConfigSectionMap('settings')['searchatlaunch']
             self.searchcolorsconf = ConfigSectionMap('settings')['colors']
+            self.searchatlaunchconf = ConfigSectionMap('settings')['searchatlaunch']
+            self.alsadeviceconf = ConfigSectionMap('settings')['alsadevice']
             if debug == True:
                 print(self.backendconf, self.codecconf, self.bitrateconf, \
                         self.samplerateconf, self.notificationsconf, \
-                        self.searchatlaunchconf, self.searchcolorsconf)
+                        self.searchatlaunchconf, self.searchcolorsconf, \
+                        self.alsadeviceconf)
 
         def write_config(self):
             """This method writes to configfile"""
             with open(self.configf, 'w') as configfile:
                     self.config.write(configfile)
 
-
 if __name__ == '__main__':
     app = QApplication(sys.argv)
     p = preferences()
