@@ -3,15 +3,15 @@
 # This file is part of mkchromecast.
 
 import mkchromecast.__init__
-from mkchromecast.audiodevices import *
+from mkchromecast.audio_devices import *
 from mkchromecast.cast import *
 from mkchromecast.config import *
 import mkchromecast.audio
 from mkchromecast.node import *
 from mkchromecast.preferences import ConfigSectionMap
-from mkchromecast.pulseaudio import *
+from mkchromecast.pulseaudio import create_sink, check_sink
 from mkchromecast.systray import *
-from PyQt5.QtCore import QThread, QObject, pyqtSignal, pyqtSlot
+from PyQt5.QtCore import (QThread, QObject, pyqtSignal, pyqtSlot)
 import os.path
 import pickle
 import pychromecast
@@ -41,10 +41,13 @@ class Worker(QObject):
         try:                        # This should fix the error socket.gaierror making the system tray to be closed.
             self.cc = casting()
             self.cc.initialize_cast()
+            self.cc.availablecc()
         except socket.gaierror:
             if debug == True:
                 print(colors.warning(':::Threading::: Socket error, CC set to 0'))
-            self.cc.availablecc == 0
+            pass
+        except TypeError:
+            pass
         if len(self.cc.availablecc) == 0 and tray == True:
             availablecc = []
             self.intReady.emit(availablecc)
@@ -64,7 +67,7 @@ class Player(QObject):
     @pyqtSlot()
     def _play_cast_(self):
         if os.path.exists(configf):
-            print(colors.warning(':::Threading::: Configuration file exist'))
+            print(colors.warning(':::Threading::: Configuration file exists'))
             print(colors.warning(':::Threading::: Using defaults set there'))
             config.read(configf)
             backend = ConfigSectionMap('settings')['backend']
@@ -82,7 +85,9 @@ class Player(QObject):
                 reload(mkchromecast.audio)
             mkchromecast.audio.main()
         if platform == 'Linux':
-            create_sink()
+            if check_sink() == False:   # We create the sink only if it is not available
+                create_sink()
+
         start = casting()
         start.initialize_cast()
         try:
@@ -130,5 +135,7 @@ class Updater(QObject):
                     self.updateready.emit('False')
             except UnboundLocalError:
                 self.updateready.emit('error1')
+            except requests.exceptions.ConnectionError:
+                self.updateready.emit('error1')
 
         self.upcastfinished.emit()
